from path import path
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.textgame import TextGame
import pygame
import threading
import time

def format_freq(f):
    if f > 1000000:
        f = f / 1000000.0
        suffix = 'MHz'
    elif f > 1000:
        f = f / 1000.0
        suffix = 'kHz'
    else:
        suffix = 'Hz'
    f = ('%f' % f).rstrip('0').rstrip('.')
    return f + '' + suffix
        
class InfoGame(CompositeGame):
    def __init__(self, avr):
#        firmware = avr.firmware
        cycle_measure_time = 1
        self.speed = 0
        self.start = pygame.time.get_ticks()
        self.firmware_time= '??:??:??'
        self.avr= avr
        
        self.reload()
        
        def avr_state():
            try:
                return avr.states[avr.state]
            except:
                return str(avr.state)
            
        CompositeGame.__init__(self,
                [
                 TextGame((lambda : "mcu=%s" % avr.mcu)),

                 TextGame((lambda : "f_cpu=%s" % format_freq(avr.f_cpu))),
                 TextGame((lambda : "%s (%s)" % (avr.firmware.filename.name
                           ,self.firmware_time))),
                 TextGame((lambda : 'prog: %s bytes %s%% ' % (avr.avrsize.program_bytes, avr.avrsize.program_percentage))),
                 TextGame((lambda : 'mem: %s bytes %s%%' % (avr.avrsize.data_bytes, avr.avrsize.data_percentage))),
                 TextGame((lambda : "vcc=%sV avcc=%sV" % (avr.vcc, avr.avcc))),
                 TextGame((lambda : "pc=%8d" % avr.pc)),
                 TextGame((lambda : "state=%s" % avr_state())),
                 TextGame((lambda : "cycle= %9d" % (avr.cycle))),
                 TextGame((lambda : "mcu time=%s us" % str(1000000 * avr.cycle / avr.f_cpu).rjust(10))),
                 TextGame((lambda : "real time=%s s" % str((pygame.time.get_ticks() - self.start) / 1000).rjust(3))),
                 TextGame((lambda : "real speed= %fx" % (self.speed))),
                 ]
                  , align=1),

        def target():
            c_old = 0
            t_old = 0
            while not self._stop_thread:
                c = avr.cycle
                t = pygame.time.get_ticks()
                
                c_diff = c - c_old
                t_diff = t - t_old
                c_old = c
                t_old = t
                
                if t_diff:
                    self.speed = 1000.0 * c_diff / t_diff / avr.f_cpu
                
                time.sleep(cycle_measure_time)
        
        self._thread = threading.Thread(target=target)
#        self._thread.daemon = 1
        self._stop_thread = False
        self._thread.start()
        
    def reload(self):
        x='??:??:??'
        if self.avr.firmware.filename.exists():
            x = time.strftime("%H:%M:%S", time.localtime(self.avr.firmware.filename.mtime))
        self.firmware_time= x
            
       
        
    def exit(self):
        self._stop_thread = 1
        CompositeGame.exit(self)
       



from entrypoint2 import entrypoint
from pysimavrgui.maingame import MainGame
from pysimavrgui.sgm7game import Sgm7Game


@entrypoint
def start():
    def func(x):
        return [
                (255,0),
                (0,0),
                (255,33),
                (7,0),
                ][x]
    dev = Sgm7Game(func,4)
    MainGame(dev).run_game()




import pygame

class TextGame(pygame.Surface):
    ''' multi line is not supported!'''
    colors = dict(
                text=(0, 0, 0),
#                transparent=(7, 7, 7),
                )
    def __init__(self, text_func, size=(30, 30), font_size=19):
        self._text_func = text_func
        self._size = size
        self.font_size=font_size
        self._font = None
#        self._surface = pygame.Surface(self.size)
#        self.surface.set_colorkey(self.colors['transparent'])
    
    @property
    def size(self):
        return self.font.size(self.text)
    
    @property
    def surface(self):
        text = self.font.render(self.text, 1, self.colors['text'])        
        return text
    
    @property
    def font(self):
        if not self._font:
            pygame.font.init()
            self._font = pygame.font.SysFont("Courier New", self.font_size)
        return self._font

    @property
    def text(self):
        if callable(self._text_func):
            s= self._text_func()
        else:
            s= self._text_func
        # show last line
#        if s:
#            s=s.splitlines()[-1]
        return s
        
     
    def update(self):
#        self.surface.fill(self.colors['transparent'])        
#        text = self.font.render(self.text, 1, self.colors['text'])        
#        self.surface.blit(text, text.get_rect().topleft)
        pass
    



from path import path
import logging
import pygame

log = logging.getLogger(__name__)

FONT_SIZE_W = 5
FONT_SIZE_H = 7
FONT_BORDER_W = 1
FONT_BORDER_H = 1

BORDER_W = 10
BORDER_H = 10

SCALE_X = 5
SCALE_Y = 5

class LcdGame(pygame.Surface):
    colors = dict(
                border=(0, 0, 0),
                font_bgr=(120, 240, 120),
                bgr=(100, 240, 100),
                text=(0, 0, 0),
#                transparent=(7, 7, 7),
                )
    def __init__(self, char_func, disp_size=(10, 2) , label=''):
        self.fonts = []
        self.disp_size = disp_size
        self.char_func = char_func
#        self.label = label
        self.disp_size = disp_size
        log.debug('creating surface:' + str(self.size))
        self._surface = pygame.Surface(self.size)
        self.load_fonts()
        
    @property
    def surface(self):
        return self._surface

    @property
    def size(self):
        w = (BORDER_W + self.disp_size[0] * (FONT_SIZE_W + FONT_BORDER_W) * SCALE_X + BORDER_W)
        h = (BORDER_H + self.disp_size[1] * (FONT_SIZE_H + FONT_BORDER_H) * SCALE_Y + BORDER_H)
        return (w, h)

    def load_fonts(self):    
        p=path(__file__).parent / "font.bmp"
        p=p.abspath()
        bmp = pygame.image.load(p);
        bmp.get_rect()
        
        for i in range(255):
            r = pygame.Rect(i * FONT_SIZE_W,
                  0,
                  FONT_SIZE_W,
                  FONT_SIZE_H)
            f = pygame.Surface((r.width, r.height))
            f.blit(bmp, (0, 0), r)
            f = pygame.transform.scale(f, (SCALE_X * FONT_SIZE_W,
                                           SCALE_Y * FONT_SIZE_H))
            f.set_colorkey((255, 255, 255))

            self.fonts += [f]
    
    def update(self):
        self.surface.fill(self.colors['bgr'])        
        pygame.draw.rect(self.surface,
                         self.colors['border'],
                         (0, 0) + self.size,
                         1
                         )   

        for v in range(self.disp_size[1]):
            for i in range(self.disp_size[0]):
                x = BORDER_W + i * (FONT_SIZE_W + FONT_BORDER_W) * SCALE_X;
                y = BORDER_H + v * (FONT_SIZE_H + FONT_BORDER_H) * SCALE_Y;
                w = FONT_SIZE_W * SCALE_X;
                h = FONT_SIZE_H * SCALE_Y;
                index = ord(self.char_func(i, v))
                pygame.draw.rect(self.surface,
                                 self.colors['font_bgr'],
                                 (x, y, w, h)
                                 )   
                f = self.fonts[index]
                self.surface.blit(f, (x, y))




from entrypoint2 import entrypoint
from pysimavrgui.lcdgame import LcdGame
from pysimavrgui.maingame import MainGame

@entrypoint
def start():
    def char_func(x,y):
        return chr(ord('a')+x+y)
    lcd = LcdGame(char_func,(11,4))
    MainGame(lcd).run_game()



from entrypoint2 import entrypoint
from path import path
from pysimavr.ac import Ac
from pysimavr.avr import Avr
from pysimavr.connect import connect_pins_by_rule
from pysimavrgui.examples.sim.avrsimmain import AvrSimMain
from pysimavr.firmware import Firmware
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.infogame import InfoGame
from pysimavrgui.lcdgame import LcdGame
from pysimavrgui.ledrowgame import LedRowGame
from pysimavr.lcd import Lcd
from pysimavr.ledrow import LedRow
from pysimavr.vcdfile import VcdFile

@entrypoint
def run_sim(vcdfile='lcd.vcd', speed=0.1, fps=20, timeout=0.0, visible=1, image_file=''):
    firmware = Firmware(path(__file__).dirname() / 'lcd.elf')
    avr = Avr(firmware,f_cpu=16000000)
    lcd = Lcd(avr)
    ledrow = LedRow(avr, size=7)
    # period=1000 -> vcd error
    vcd = VcdFile(avr, period=10, filename=vcdfile)
    
    def state_func(i):
        return (ledrow.pinstate(i), ledrow.reset_dirty(i))
    led_game = LedRowGame(state_func=state_func,
                         labels='D4 D5 D6 D7 RS E RW'.split()  
                         )
    ac = Ac(avr)
    connect_pins_by_rule('''
    avr.B0 <=> lcd.D4 -> vcd
    avr.B1 <=> lcd.D5 -> vcd
    avr.B2 <=> lcd.D6 -> vcd
    avr.B3 <=> lcd.D7 -> vcd
    
    avr.B4 ==> lcd.RS -> vcd
    avr.B5 ==> lcd.E  -> vcd
    avr.B6 ==> lcd.RW -> vcd
    vcd <- ac.OUT -> avr.D2

    lcd.D4 -> led.0
    lcd.D5 -> led.1
    lcd.D6 -> led.2
    lcd.D7 -> led.3
    
    lcd.RS -> led.4
    lcd.E  -> led.5
    lcd.RW -> led.6
                        ''',
                         dict(
                             avr=avr,
                             led=ledrow,
                             lcd=lcd,
                             ac=ac
                             ),
                         vcd=vcd,
                         )
        
    dev = CompositeGame([
                      CompositeGame(
                              [LcdGame(lambda x, y:lcd.get_char(x, y), (20, 2)),
                               led_game,
                               ], 
                                align=1),
                      InfoGame(avr),
                      ])
    
    scrshot_by_exit = [(dev, image_file)] if image_file else None
    
    AvrSimMain(avr, dev, vcd, speed=speed, fps=fps, visible=visible, timeout=timeout,
               scrshot_by_exit=scrshot_by_exit).run_game()




import logging
import pygame

log = logging.getLogger(__name__)

FONT_SIZE_W = 5
FONT_SIZE_H = 7
FONT_BORDER_W = 1
FONT_BORDER_H = 1

BORDER_W = 10
BORDER_H = 10

SCALE_X = 5
SCALE_Y = 5

SEGMENT_A = (1 << 0)
SEGMENT_B = (1 << 1)
SEGMENT_C = (1 << 2)
SEGMENT_D = (1 << 3)
SEGMENT_E = (1 << 4)
SEGMENT_F = (1 << 5)
SEGMENT_G = (1 << 6)
SEGMENT_P = (1 << 7)
ALL_SEGMENTS = SEGMENT_A | SEGMENT_B | SEGMENT_C | SEGMENT_D | SEGMENT_E | SEGMENT_F | SEGMENT_G | SEGMENT_P

class Sgm7Game(pygame.Surface):
    colors = dict(
                  on=(0, 0, 0),
                off=[230]*3,
                pulse=(222, 0, 0),
                border=(0, 0, 0),
#                sgm_bg=(210, 210, 210),
#                sgm_fg=(255, 0, 0),
                bgr=[210]*3,
                text=(0, 0, 0),
#                transparent=(7, 7, 7),
                )
    def __init__(self, segments_func, disp_size=4 , label=''):
        self.fonts = []
        self.disp_size = disp_size
        self.segments_func = segments_func
#        self.label = label
        log.debug('creating surface:' + str(self.size))
        self._surface = pygame.Surface(self.size)
        
    @property
    def surface(self):
        return self._surface

    @property
    def size(self):
        w = (BORDER_W + self.disp_size * (FONT_SIZE_W + FONT_BORDER_W) * SCALE_X + BORDER_W)
        h = (BORDER_H + (FONT_SIZE_H + FONT_BORDER_H) * SCALE_Y + BORDER_H)
        return (w, h)

    def update(self):
        self.surface.fill(self.colors['bgr'])        
        pygame.draw.rect(self.surface,
                         self.colors['border'],
                         (0, 0) + self.size,
                         1
                         )   
        for i in range(self.disp_size):
            (on,reset)=self.segments_func(i)
            self.draw_digit(i, ALL_SEGMENTS, self.colors['off'])
            self.draw_digit(i, on, self.colors['on'])
            self.draw_digit(i, reset, self.colors['pulse'])

    def draw_digit(self, i, segments, color):
        v = 0
        x = BORDER_W + i * (FONT_SIZE_W + FONT_BORDER_W) * SCALE_X
        y = BORDER_H + v * (FONT_SIZE_H + FONT_BORDER_H) * SCALE_Y
        w = FONT_SIZE_W * SCALE_X
        h = FONT_SIZE_H * SCALE_Y
    
        segm_w = w / 7
        segm_h = w / 7
        edge_w = w / 9
        edge_h = w / 9
    
        x1l = x + edge_w
        x2l = x + w - edge_w - segm_w
        x1r = x1l + segm_w
        x2r = x2l + segm_w
    
        y1t = y + edge_h
        y2t = y + h / 2 - segm_h / 2
        y3t = y + h - edge_h - segm_h
        y1b = y1t + segm_h
        y2b = y2t + segm_h
        y3b = y3t + segm_h
    
        if segments & SEGMENT_P:
            pos = x2r + 1.5 * segm_w, y3t
            r = segm_w
            pygame.draw.circle(self.surface, color, map(int,pos), int(r))   
        
    
        if segments & SEGMENT_E:    
            r = x1l, y2t, x1r - x1l, y3b - y2t
            pygame.draw.rect(self.surface, color, r)   
        
    
        if segments & SEGMENT_C:
            r = x2l, y2t, x2r - x2l, y3b - y2t
            pygame.draw.rect(self.surface, color, r)   
        
    
        if segments & SEGMENT_F:
            r = x1l, y1t, x1r - x1l, y2b - y1t
            pygame.draw.rect(self.surface, color, r) 
        
        if segments & SEGMENT_B:
            r = x2l, y1t, x2r - x2l, y2b - y1t
            pygame.draw.rect(self.surface, color, r) 
        
        if segments & SEGMENT_A:
            r = x1l, y1t, x2r - x1l, y1b - y1t
            pygame.draw.rect(self.surface, color, r) 
        
        if segments & SEGMENT_G:
            r = x1l, y2t, x2r - x1l, y2b - y2t
            pygame.draw.rect(self.surface, color, r) 
        
        if segments & SEGMENT_D:
            r = x1l, y3t, x2r - x1l, y3b - y3t
            pygame.draw.rect(self.surface, color, r) 
        




'''

lcd:

#include <LiquidCrystal.h>

LiquidCrystal lcd(12, 11, 10, 5, 4, 3, 2);

void setup() {
  lcd.begin(16, 2);
  lcd.println("hello, world!");
}

void loop() {
}

connect UDP with netcat:
nc -u 127.0.0.1 4321

'''
from entrypoint2 import entrypoint
from path import path
from pysimavr.avr import Avr
from pysimavr.button import Button
from pysimavr.connect import connect_pins_by_rule
from pysimavr.firmware import Firmware
from pysimavr.lcd import Lcd
from pysimavr.ledrow import LedRow
from pysimavr.spk import Spk
from pysimavr.udp import Udp
from pysimavr.udpreader import UdpReader
from pysimavr.vcdfile import VcdFile
from pysimavrgui.buttongame import ButtonGame
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.examples.sim.avrsimmain import AvrSimMain
from pysimavrgui.infogame import InfoGame
from pysimavrgui.lcdgame import LcdGame
from pysimavrgui.ledrowgame import LedRowGame
from pysimavrgui.spkgame import SpkGame
from pysimavrgui.textgame import TextGame
import os
import sys
import tempfile
import time
try:
    import pyavrutils
except ImportError:
    pyavrutils = None


        

def lastline(s):
    if s:
        s = s.splitlines(1)[-1]        
    return s

def find_elf():
    build_dirs = [x for x in path(tempfile.gettempdir()).dirs('build*.tmp')]
    
    elfs = []
    
    last = path(os.getenv("HOME")) / '.arduino_last_elf'
    if not last.exists():
        last.makedirs()
    elfs += last.files()
        
    for d in build_dirs:
        elfs += [x for x in d.files('*.elf')] 
    elfs.sort(key=path.getmtime, reverse=1) 

    assert len(elfs), 'No elf file found in "%s"! Start arduino and build a sketch!' % tempfile.gettempdir()
    elf = elfs[0]
    
#    if last == elf and len(elfs) > 1:
#        if last.mtime == elfs[1].mtime:
#            elf = elfs[1]
    
    # save last elf
    if elf.parent != last:
        elf.copy(last / elf.name)
    
    return elf


           
@entrypoint
def arduino_sim(
            elf='',
            mcu='atmega168',
            f_cpu=16000000,
            vcdfile='arduino.vcd',
            speed=1,
            fps=20,
            timeout=0.0,
            visible=1,
            image_file='',
            rate=11025,
            buttons_enable=1,
            vcd_enable=0,
            spk_enable=0,
            udp_enable=1,
            avcc=5000,
            vcc=5000,
            code=None,
            ):
    '''
    
    MCU:
     - atmega168 OK 
     - atmega328p OK
     - atmega2560 NO
     - atmega1280 NO
     
    :param mcu: 
    :param avcc: AVcc  in mV
    :param vcc: Vcc  in mV
    
    '''

    if code and pyavrutils:
        cc = pyavrutils.Arduino()
        cc.build(code)
        elf = cc.output
   
    if not elf:
        elf = find_elf()
    firmware = Firmware(elf)
    avr = Avr(mcu=mcu, f_cpu=f_cpu, vcc=vcc / 1000.0, avcc=avcc / 1000.0)
    avr.load_firmware(firmware)
    
    udpReader = UdpReader()
    
    if udp_enable:
        udp = Udp(avr)
        udp.connect()
        udpReader.start()
        
    lcd = Lcd(avr)

    vcd = VcdFile(avr, period=1000, filename=vcdfile) if vcd_enable else None
    ledrow = LedRow(avr, size=14)
    buttons = [Button(avr, pullup=0) for x in range(14)] if buttons_enable else 14 * [None]
    spk = Spk(avr, rate=rate, speed=speed) if spk_enable else None
    
    
    
    connect_pins_by_rule('''
                        but0 .OUT ==> avr.D0 ==> dig.0  -> vcd
                        but1 .OUT ==> avr.D1 ==> dig.1  -> vcd
                        but2 .OUT ==> avr.D2 ==> dig.2  -> vcd
                        but3 .OUT ==> avr.D3 ==> dig.3  -> vcd
                        but4 .OUT ==> avr.D4 ==> dig.4  -> vcd
                        but5 .OUT ==> avr.D5 ==> dig.5  -> vcd
                        but6 .OUT ==> avr.D6 ==> dig.6  -> vcd
                        but7 .OUT ==> avr.D7 ==> dig.7  -> vcd

                        but8 .OUT ==> avr.B0 ==> dig.8  -> vcd
                        but9 .OUT ==> avr.B1 ==> dig.9  -> vcd
                        but10.OUT ==> avr.B2 ==> dig.10 -> vcd
                        but11.OUT ==> avr.B3 ==> dig.11 -> vcd
                        but12.OUT ==> avr.B4 ==> dig.12 -> vcd
                        but13.OUT ==> avr.B5 ==> dig.13 -> vcd

                        dig.13 --> spk.IN -> vcd
                        
                        dig.5 <=> lcd.D4 -> vcd
                        dig.4 <=> lcd.D5 -> vcd
                        dig.3 <=> lcd.D6 -> vcd
                        dig.2 <=> lcd.D7 -> vcd
                        
                        dig.12 ==> lcd.RS -> vcd
                        dig.11 ==> lcd.RW -> vcd
                        dig.10 ==> lcd.E  -> vcd
                        ''',
                         dict(
                             avr=avr,
                             dig=ledrow,
                             lcd=lcd,
                             spk=spk,
                             **dict([('but' + str(i), b) for i, b in enumerate(buttons)])
                             ),
                         vcd=vcd,
    )
    #################
    # GUI
    #################
    if spk_enable:
        def spk_func(size):
            return spk.read()
        spk_game = SpkGame(spk_func, rate=rate)

    def state_func(i):
        return (ledrow.pinstate(i), ledrow.reset_dirty(i))
    led_game = LedRowGame(state_func=state_func,
                          labels=[str(x) for x in range(14)],
                          align=1,)
        
    but_guis = [ButtonGame(
                              label=label,
                              shortcut=shortcut,
                              hook=dict(up=x.up, down=x.down) if x else {},
                              size=(50, 30),
                              ) for x, label, shortcut in zip(
                                             buttons,
                                             '0 1 2 3 4 5 6 7 8 9 10 11 12 13'.split(),
                                             '0 1 2 3 4 5 6 7 8 9 a b c d'.split(),
                                             )
                   ]
    info = InfoGame(avr)
    def reload_firmware():
        firmware = Firmware(find_elf())
        avr.load_firmware(firmware)
        lcd.reset()
        info.reload()
        
    class MyFloat(object):
        def __init__(self, value=0.0):
            self.value = value
        def __float__(self):
            return float(self.value)
        def inc(self):
            self.value *= 10.0
        def dec(self):
            self.value /= 10.0
    speed = MyFloat(speed)
    
    def udp_read():
        if not hasattr(udp_read, 'display'):
            udp_read.display = ''
        if not hasattr(udp_read, 'lastline'):
            udp_read.lastline = ''
        s = udpReader.read()
        if s:
            sys.stdout.write(s)
            udp_read.lastline += s
            udp_read.lastline = lastline(udp_read.lastline)
            udp_read.display = udp_read.lastline.replace('\n', '\\n').replace('\r', '\\r')
        return  udp_read.display
    
    dev = CompositeGame([
                        CompositeGame([led_game,
                                       CompositeGame(but_guis, align=1),
                                       ], align=0),
                        info,
                        CompositeGame([
                            CompositeGame([
                                ButtonGame(label='reload',
                                           shortcut='r',
                                           hook=dict(down=reload_firmware)),
                                ButtonGame(label='speed up',
                                           shortcut='+',
                                           hook=dict(down=speed.inc)),
                                ButtonGame(label='speed down',
                                           shortcut='-',
                                           hook=dict(down=speed.dec)),
                                TextGame((lambda : "speed set= %fx" % float(speed))),
                                           ], align=1),
                                       
                                LcdGame(lambda x, y:lcd.get_char(x, y), (16, 2)),
                                TextGame((lambda : 'ser=' + udp_read())),
                                ], align=1)
                       ])

    scrshot_by_exit = [(dev, image_file)] if image_file else None
    
    class ArduinoMain(AvrSimMain):
        bufpos = 0
        def cb_loop(self):
            AvrSimMain.cb_loop(self)
#            if len(udpReader.buffer) > self.bufpos:
#                sys.stdout.write(''.join(udpReader.buffer[self.bufpos:]))
#                self.bufpos = len(udpReader.buffer)

    sim = ArduinoMain(avr, dev, vcd, speed=speed, fps=fps, visible=visible, timeout=timeout,
               scrshot_by_exit=scrshot_by_exit)
    sim.run_game()
    
    time.sleep(1)
    if spk_enable:
        spk_game.terminate()

    if udp_enable:
        udp.terminate()
        
    if udp_enable:
        udpReader.terminate()
#        return udpReader.buffer




from entrypoint2 import entrypoint
from path import path
from pysimavr.ac import Ac
from pysimavr.avr import Avr
from pysimavr.connect import connect_pins_by_rule
from pysimavr.firmware import Firmware
from pysimavr.ledrow import LedRow
from pysimavr.spk import Spk
from pysimavr.vcdfile import VcdFile
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.examples.sim.avrsimmain import AvrSimMain
from pysimavrgui.infogame import InfoGame
from pysimavrgui.ledrowgame import LedRowGame
from pysimavrgui.spkgame import SpkGame

@entrypoint
def run_sim(vcdfile='spk.vcd', speed=0.5, fps=20, timeout=0.0, visible=1, image_file='',rate=11025):
    firmware = Firmware(path(__file__).dirname() / 'spk.elf')
    avr = Avr(firmware=firmware,
              mcu="atmega168",
              f_cpu=16000000,
              )
    ledrow = LedRow(avr, size=1)
    # period=1000 -> vcd error
    vcd = VcdFile(avr, period=10, filename=vcdfile)
    spk = Spk(avr, rate=rate, speed=speed)
    
    connect_pins_by_rule('''
    led.0 <-- avr.B5 --> spk.IN -> vcd
                        ''',
                         dict(
                             avr=avr,
                             spk=spk,
                             led=ledrow,
                             ),
                         vcd=vcd,)
    ####################################                     )
    # GUI        
    def spk_func(size):
        return spk.read()
    spk_game = SpkGame(spk_func, rate=rate)

    def state_func(i):
        return (ledrow.pinstate(i), ledrow.reset_dirty(i))
    led_game = LedRowGame(state_func=state_func,
                         labels=['SPK']  
                         )
    dev = CompositeGame([
                      CompositeGame(
                              [
                               led_game,
                               ],
                                align=1),
                      InfoGame(avr),
                      ])
    
    scrshot_by_exit = [(dev, image_file)] if image_file else None
    
    AvrSimMain(avr, dev, vcd, speed=speed, fps=fps, visible=visible, timeout=timeout,
               scrshot_by_exit=scrshot_by_exit).run_game()

    spk_game.terminate()



import logging
import threading

try:
    import pyaudio
except ImportError:
    logging.debug('pyaudio is missing, sound is not supported')
    
chunk = 1024

class SpkGame():
    ''' sound is very poor'''
    def __init__(self, func, rate=11025):
        ''
        self.func = func
        self.rate = rate
        self._stop_thread = False
        self._thread = None
        if pyaudio:
            self.start()
            
    def start(self):
        self.p = pyaudio.PyAudio()

        # open stream
        self.stream = self.p.open(format=self.p.get_format_from_width(1),
                        channels=1,
                        rate=self.rate,
                        output=True)

        def target():
            while not self._stop_thread:
                # play stream
                data = self.func(chunk)
                if len(data):
                    self.stream.write(data)

        self._thread = threading.Thread(target=target)
        self._thread.start()


    def terminate(self):
        if pyaudio:
            self._stop_thread = True
            self._thread.join()
            self.stream.close()
            self.p.terminate()
        



import pygame
import pygame.gfxdraw

class LedGame(pygame.Surface):
    colors = dict(
                  on=(255, 100, 100),
                off=(100, 255, 100),
                pulse=(175, 175, 100),
                border=(0, 0, 0),
                text=(0, 0, 0),
                transparent=(7, 7, 7),
                )
    def __init__(self, state_func, label='', size=(30, 30)):
        self._state_func = state_func
        self.label = label
        self._size = size
        self._surface = pygame.Surface(size)
        edge = 2
        self.pos = (self.size[0] / 2, self.size[1] / 2)
        self.r = min(self.size) / 2 - edge
        self.font = None
        self._bg=self._surface.copy()
#        self._bg.set_alpha(1)
        self._bg.fill(self.colors['transparent'])     
#        self._bg.set_colorkey(self.colors['transparent'])
        self.surface.set_colorkey(self.colors['transparent'])
    
    @property
    def size(self):
        return self._size
    
    @property
    def surface(self):
        return self._surface

    @property
    def state(self):
        ' (on/off,pulse)'
        if callable(self._state_func):
            return self._state_func()
        else:
            return self._state_func
        
    @property
    def on(self):
        return self.state[0]

    @property
    def pulse(self):
        return self.state[1]
    
    def update(self):
        if not self.font:
            self.font = pygame.font.SysFont("Courier New", self.r)
#        self.surface.fill(self.colors['transparent'])     
        self.surface.blit(self._bg, (0,0))
   
#        pygame.draw.circle(self.surface,
#                           self.colors[self.state],
#                           self.pos,
#                           self.r,
#                           )
        if self.pulse:
            color=self.colors['pulse']
        elif self.on:
            color=self.colors['on']
        else:
            color=self.colors['off']
        pygame.gfxdraw.filled_circle(self.surface,
                                     self.pos[0], self.pos[1], self.r ,
                                     color)
        if self.label:
            text = self.font.render(self.label, 1, self.colors['text'])        
            self.surface.blit(text, text.get_rect(center=self.pos).topleft)
        pygame.gfxdraw.aacircle(self.surface,
                                     self.pos[0], self.pos[1], self.r ,
                                     self.colors['border'])
    



from entrypoint2 import entrypoint
from path import path
from pysimavr.avr import Avr
from pysimavr.connect import connect_pins_by_rule
from pysimavrgui.examples.sim.avrsimmain import AvrSimMain
from pysimavr.firmware import Firmware
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.infogame import InfoGame
from pysimavrgui.ledrowgame import LedRowGame
from pysimavr.ledrow import LedRow
from pysimavr.vcdfile import VcdFile

@entrypoint
def run_sim(vcdfile='ledramp.vcd', speed=0.1, fps=20, timeout=0.0, visible=1, image_file=''):
    firmware = Firmware(path(__file__).dirname() / 'ledramp.elf')
    avr = Avr(firmware, f_cpu=8000000, mcu='atmega48')

    vcd = VcdFile(avr, period=1000, filename=vcdfile)
    ledrow = LedRow(avr)
    connect_pins_by_rule('''
                        avr.B0 ==> led.0 -> vcd
                        avr.B1 ==> led.1 -> vcd
                        avr.B2 ==> led.2 -> vcd
                        avr.B3 ==> led.3 -> vcd
                        avr.B4 ==> led.4 -> vcd
                        avr.B5 ==> led.5 -> vcd
                        avr.B6 ==> led.6 -> vcd
                        avr.B7 ==> led.7 -> vcd
                        ''',
                         dict(
                             avr=avr,
                             led=ledrow,
                             ),
                         vcd=vcd,
    )
    
    def state_func(i):
        return (ledrow.pinstate(i), ledrow.reset_dirty(i))
    led_game = LedRowGame(state_func=state_func,
                          labels=['B' + str(x) for x in range(8)])
        
    dev = CompositeGame([
                        led_game,
                        InfoGame(avr),
                        ])
    
    scrshot_by_exit = [(dev, image_file)] if image_file else None

    AvrSimMain(avr, dev, vcd, speed=speed, fps=fps, visible=visible, timeout=timeout,
               scrshot_by_exit=scrshot_by_exit).run_game()



from entrypoint2 import entrypoint
from pysimavrgui.buttongame import ButtonGame
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.maingame import MainGame

@entrypoint
def start():
    def func_up1():
        print 'up1'
    def func_down1():
        print 'down1'
    def func_up2():
        print 'up2'
    def func_down2():
        print 'down2'
    dev1 = ButtonGame(hook=dict(up=func_up1, down=func_down1), label='1', shortcut='1')
    dev2 = ButtonGame(hook=dict(up=func_up2, down=func_down2), label='2', shortcut='2')

    dev=CompositeGame([dev1,dev2])
    MainGame(dev).run_game()




from pysimavrgui.maingame import MainGame
import logging

log = logging.getLogger(__name__)

class AvrSimMain(MainGame):
    def __init__(self, avr, dev, vcd, timeout=None, visible=1, speed=1, fps=20, scrshot_by_exit=None):
        log.debug('visible=%s timeout=%s speed=%s'%(visible, timeout,speed))
        self.avr = avr
        self.vcd = vcd
        self.speed = speed
        self.timeout = timeout            
        MainGame.__init__(self, dev, fps=fps, visible=visible, scrshot_by_exit=scrshot_by_exit)
        
    def run_game(self):
        if self.vcd:
            self.vcd.start()
        MainGame.run_game(self)
            
    def cb_loop(self):
        ''
        self.avr.move_time_marker(1.0 * float(self.speed) / self.fps)
        if self.timeout:
#            self.avr.goto_time(self.timeout)
            if self.avr.time_passed() >= self.timeout:
                log.debug('avr.time_passed=%d' % self.avr.time_passed())
                log.debug('timeout=%d' % self.timeout)
                log.debug('exiting')
                self.exit = True               
            
    def cb_exit(self):
        if self.vcd:
            self.vcd.terminate()
        self.avr.terminate()






from path import path
import logging
import pygame

log = logging.getLogger(__name__)

class MainGame(object):
#    BG_COLOR = 150, 150, 80
    BG_COLOR = 100, 100, 180
    def __init__(self, dev, pos=(0, 0), fps=50, size='auto', title='python-simavr', visible=True, scrshot_by_exit=None):
        self.scrshot_by_exit = scrshot_by_exit
        self.visible = visible
        self.fps = fps
        self.size = size
        self.dev = dev
        self.pos = pos
        self.title = title
        self.exit = False               
        self.exception = None
               
    def cb_loop(self):
        pass
    
    def cb_exit(self):
        pass
    
    def screenshot(self, dev=None, img_file='screenshot.png'):
        if not dev:
            dev = self.dev
        img_file = path(img_file).abspath()
        log.debug('saving ' + str(img_file))
        pygame.image.save(dev.surface, img_file)

    def run_game(self):
        pygame.init() 
        if self.visible:
            pygame.display.set_caption(self.title) 
        clock = pygame.time.Clock()
        if self.size == 'auto':
            size = self.dev.size
            
        if self.visible:
            #create the screen
            self.window = pygame.display.set_mode(size, 0)
        else:  
            self.window = pygame.Surface(size)

        
        log.debug('driver:' + pygame.display.get_driver())
        while not self.exit:
            # Redraw the background
            self.window.fill(self.BG_COLOR)     
            
            self.dev.abs_pos = self.pos
            self.dev.update()            
            self.window.blit(self.dev.surface, self.pos)
            
            if self.visible:            
                #draw it to the screen
                pygame.display.flip()
    
    
            # Limit frame speed
            self.time_passed = clock.tick(self.fps) 
            real_fps = (clock.get_fps())
            pygame.display.set_caption(
                      '{title} (fps={real_fps})'.format(
                                        title=self.title,
                                        real_fps=int(0.5 + real_fps)))

            self.cb_loop()
            try:
                self.handleEvents()
            except Exception as e:
                self.exception = e
                break    
#            for event in pygame.event.get():
#                if event.type == pygame.QUIT:                    
#                    self.exit = True
        self.terminate()
        
    def handleEvents(self):
        exit = False
        for event in pygame.event.get():
            if event.type == pygame.QUIT:                    
                self.exit = True
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.exit = True
            if hasattr(self.dev, 'handleEvents'):
                self.dev.handleEvents(event)
        
    def terminate(self):   
        if self.scrshot_by_exit:
            for dev, img_file in self.scrshot_by_exit:
                self.screenshot(dev, img_file)
        self.cb_exit()
        if hasattr(self.dev, 'exit'):
            self.dev.exit()
#        pygame.quit()
#        sys.exit()
        if                 self.exception:
            raise                 self.exception 







from entrypoint2 import entrypoint
from path import path
from pysimavr.avr import Avr
from pysimavr.connect import connect_pins_by_rule
from pysimavrgui.examples.sim.avrsimmain import AvrSimMain
from pysimavr.firmware import Firmware
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.infogame import InfoGame
from pysimavrgui.ledrowgame import LedRowGame
from pysimavrgui.sgm7game import Sgm7Game
from pysimavr.inverter import Inverter
from pysimavr.ledrow import LedRow
from pysimavr.sgm7 import Sgm7
from pysimavr.vcdfile import VcdFile

@entrypoint
def run_sim(vcdfile='sgm7.vcd', speed=0.001, fps=20, timeout=0.0, visible=1, image_file=''):
    firmware = Firmware(path(__file__).dirname() / 'sgm7.elf')
    firmware.f_cpu = 8000000
    firmware.mcu = "atmega168"
    avr = Avr(firmware)
    vcd = VcdFile(avr, period=1000, filename=vcdfile)

    ####################################################
    # ledrow
    ledrow = LedRow(avr, size=12)
    
    ####################################################
    # ledrow game
    def state_func_seg(i):
        return (ledrow.pinstate(i), ledrow.reset_dirty(i))
    led_game_seg = LedRowGame(state_func=state_func_seg,
                         disp_size=8,
                         labels=['B' + str(x) for x in range(8)]  
                         )
    def state_func_dig(i):
        return (ledrow.pinstate(i + 8), ledrow.reset_dirty(i + 8))
    led_game_dig = LedRowGame(state_func=state_func_dig,
                         disp_size=4,
                         labels=['C' + str(x) for x in range(4)])
    ####################################################
    # sgm7
    sgm7 = Sgm7(avr, size=4)

    inv = [Inverter(avr) for x in range(4)]
    
    connect_pins_by_rule('''
                        ledrow.0 <== avr.B0 ==> sgm7.A -> vcd
                        ledrow.1 <== avr.B1 ==> sgm7.B -> vcd
                        ledrow.2 <== avr.B2 ==> sgm7.C -> vcd
                        ledrow.3 <== avr.B3 ==> sgm7.D -> vcd
                        ledrow.4 <== avr.B4 ==> sgm7.E -> vcd
                        ledrow.5 <== avr.B5 ==> sgm7.F -> vcd
                        ledrow.6 <== avr.B6 ==> sgm7.G -> vcd
                        ledrow.7 <== avr.B7 ==> sgm7.P -> vcd
                        
                        ledrow.8 <== avr.C0 ==> inv0.IN | inv0.OUT -> sgm7.D0 -> vcd
                        ledrow.9 <== avr.C1 ==> inv1.IN | inv1.OUT -> sgm7.D1 -> vcd
                        ledrow.10<== avr.C2 ==> inv2.IN | inv2.OUT -> sgm7.D2 -> vcd
                        ledrow.11<== avr.C3 ==> inv3.IN | inv3.OUT -> sgm7.D3 -> vcd
                        ''',
                         dict(
                             avr=avr,
                             sgm7=sgm7,
                             ledrow=ledrow,
                             inv0=inv[0],
                             inv1=inv[1],
                             inv2=inv[2],
                             inv3=inv[3],
                             ),
                         vcd=vcd,
                         )

    ####################################################
    # sgm7 game
    def segments_func(digit_index):
        return (sgm7.digit_segments(digit_index), sgm7.reset_dirty(digit_index))
    sgm7_game = Sgm7Game(segments_func=segments_func, disp_size=4)
         
    ####################################################
    # compose game
    dev = CompositeGame([
                      CompositeGame(
                              [
                               sgm7_game,
                               led_game_seg,
                               led_game_dig,
                               ],
                                    align=1),
                      InfoGame(avr),
                      ])
    
    scrshot_by_exit = [(dev, image_file)] if image_file else None
    
    AvrSimMain(avr, dev, vcd, speed=speed, fps=fps, visible=visible, timeout=timeout,
               scrshot_by_exit=scrshot_by_exit).run_game()




from entrypoint2 import entrypoint
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.maingame import MainGame
from pysimavrgui.textgame import TextGame

@entrypoint
def start():
    def func1():
        return 'hello'
    def func2():
        return 'hi'
    dev1 = TextGame(func1)
    dev2 = TextGame(func2)
    dev=CompositeGame([dev1,dev2],align=1)
    MainGame(dev).run_game()




from entrypoint2 import entrypoint
from pysimavrgui.ledrowgame import LedRowGame
from pysimavrgui.maingame import MainGame

@entrypoint
def start():
    def func(i):
        return (i>1,i>2)
    dev = LedRowGame(func,disp_size=4,labels=['x','y','z'])
    MainGame(dev).run_game()




import pygame
#import pygame.gfxdraw

class ButtonGame(pygame.Surface):
    colors = dict(
                  on=(255, 100, 100),
                off=(100, 255, 100),
                border=(0, 0, 0),
                text=(0, 0, 0),
                transparent=(7, 7, 7),
                )
    def __init__(self, hook=None, shortcut=None, label='', size='auto', display_shortcut=True, font_size=14):
        self.label = label
        if display_shortcut and shortcut:
            self.label += ' (%s)' % shortcut
        self.font_size = font_size
        self.shortcut = shortcut
        self.hook = hook if hook else dict()
        self.pressed = False
        self._size = size
        pygame.font.init()
        self.font = pygame.font.SysFont("Courier New", self.font_size)
        if size == 'auto':
            self._size = self.font.size(self.label)  
            self._size=(self._size[0]+10,self._size[1]+5)     
        self._surface = pygame.Surface(self.size)
        self.key = None

#        self.font = None
        self._bg = self._surface.copy()
        self._bg.fill(self.colors['transparent'])     
        self.surface.set_colorkey(self.colors['transparent'])
        
         
        edge = 2
        self.center = (self.size[0] / 2, self.size[1] / 2)
        self.rect = pygame.Rect(edge, edge, self.size[0] - edge, self.size[1] - edge)
    
    @property
    def size(self):
        return self._size
    
    @property
    def surface(self):
        return self._surface
        
    def update(self):
#        if not self.font:
        self.surface.blit(self._bg, (0, 0))
        if self.pressed:
            color = self.colors['on']
        else:
            color = self.colors['off']
        pygame.draw.rect(self.surface, color, self.rect, 0)
        pygame.draw.rect(self.surface, self.colors['border'], self.rect, 1)
        if self.label:
            text = self.font.render(self.label, 1, self.colors['text'])        
            self.surface.blit(text, text.get_rect(center=self.center).topleft)
            
    def _press(self, b):
        self.pressed = b
        f = self.hook.get('down' if b else 'up')
        if f:
            f()
            
    def handleEvents(self, event):
        abs_rect = self.rect.move(*self.abs_pos)
        if event.type == pygame.MOUSEBUTTONDOWN:
            if abs_rect.collidepoint(event.pos) and event.button == 1:
                self._press(True)
        if event.type == pygame.MOUSEBUTTONUP:
            if abs_rect.collidepoint(event.pos) and event.button == 1:
                self._press(False)
        if event.type == pygame.KEYDOWN:
            if event.unicode == self.shortcut:
                self.key = event.key
                self._press(True)
        if event.type == pygame.KEYUP:
            if event.key == self.key:
                self._press(False)



from entrypoint2 import entrypoint
from pysimavrgui.compgame import CompositeGame
from pysimavrgui.ledgame import LedGame
from pysimavrgui.maingame import MainGame

@entrypoint
def start():
    def func_on():
        return (1,0)
    def func_off():
        return (0,0)
    def func_pulse():
        return (1,1)
    dev1 = LedGame(func_on,'on')
    dev2 = LedGame(func_off,'off')
    dev3 = LedGame(func_pulse,'pulse')
    dev=CompositeGame([dev1,dev2,dev3])
    MainGame(dev).run_game()




from entrypoint2 import entrypoint
from pysimavrgui.spkgame import SpkGame
import time

@entrypoint
def start():
    data= 20*'\0'+20*'Z'
    def func(size):
        return data * (size/40)
    spk = SpkGame(func)
    time.sleep(3)
    spk.terminate()


from pysimavrgui.compgame import CompositeGame
from pysimavrgui.ledgame import LedGame



class LedRowGame(CompositeGame):
    def __init__(self, state_func, disp_size=None, labels=None, align=0, size='auto'):
        if disp_size is None and labels is None:
            raise ValueError('disp_size is None and labels is None')

        if disp_size is None:
            disp_size = len(labels)
        
        self.disp_size = disp_size
        self.leds = []
        self.state_func = state_func
        for i in xrange(self.disp_size):
            s = None
            if labels:
                if i<len(labels):
                    s = labels[i]
            led = LedGame(state_func=self._get_single_state_func(i), label=s)
            self.leds += [led]

        CompositeGame.__init__(self, self.leds, align=align, size=size)
        
    def _get_single_state_func(self, i):
        def f():
            return self.state_func(i)
        return f
    
        



import pygame

class CompositeGame(object):
#    BG_COLOR = 150, 150, 80
    BG_COLOR = 220, 220, 255
    def __init__(self, devs, align=0, size='auto', gap=2):
        self._size = size
        self.devs = devs
        self.align = align
        self.gap = gap
        self._surface = pygame.Surface(self.size)
        
    def update(self):
        self.surface.fill(self.BG_COLOR)     
        pos = [self.gap] * 2
        for x in self.devs:
            x.abs_pos = map(sum, zip(self.abs_pos , pos))
            x.update()
            self.surface.blit(x.surface, pos)
            pos[self.align] += x.size[self.align] + self.gap
            
    def exit(self):
        for x in self.devs:
            if hasattr(x, 'exit'):
                x.exit()
                
    def handleEvents(self, event):
        for x in self.devs:
            if hasattr(x, 'handleEvents'):
                x.handleEvents(event)
            
    @property
    def surface(self):
        return self._surface
    
    @property
    def size(self):
        size = [0, 0]
        devs = self.devs
        sumgap = self.gap * (len(devs) + 1)
        
        i = self.align
        size[i] = sum([d.size[i] for d in devs]) + sumgap
        
        i = 1 - self.align
        size[i] = max([d.size[i] for d in devs]) + 2 * self.gap
        
        return tuple(size)
    



import socket
import time

UDP_IP="127.0.0.1"
UDP_PORT=4321

sock = socket.socket( socket.AF_INET, # Internet
                      socket.SOCK_DGRAM ) # UDP
#sock.bind( (UDP_IP,UDP_PORT) )

sock.sendto( 'xxx', (UDP_IP, UDP_PORT) )

while True:
    data, addr = sock.recvfrom( 1024 ,MSG_WAITALL) # buffer size is 1024 bytes
    print "received message:", data
    time.sleep(1)
